#!/usr/bin/python

#
#  Copyright (C) 2004 Red Hat, Inc.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import sys
import replicator
import gobject
from config import StatelessConfig

class StatelessBootstrap (gobject.GObject):
    __gsignals__ = {
        'update'       : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_FLOAT,  )),
        'state-change' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_STRING, )),
        'finished'     : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (gobject.TYPE_BOOLEAN, ))
        }

    def __init__ (self, debug_mode = False):
        gobject.GObject.__init__ (self)
        
        self.debug_mode = debug_mode

        self.main_loop = gobject.MainLoop ()

        self.cfg = StatelessConfig ()
        self.replicator = replicator.BootstrapReplicator (self.cfg, debug_mode)

    def on_replicator_update (self, replicator, percentage):
        self.emit ('update', percentage)
    
    def on_replicator_state_change (self, replictor, state):
        self.emit ('state-change', state)

    def on_replicator_finished (self, replicator, success):
        self.emit ('finished', success)
        self.main_loop.quit ()

    def run (self):
        self.replicator.connect ('update',       self.on_replicator_update)
        self.replicator.connect ('state-change', self.on_replicator_state_change)
        self.replicator.connect ('finished',     self.on_replicator_finished)

        self.replicator.begin_update_client ()

        self.main_loop.run ()

gobject.type_register (StatelessBootstrap)

def unmount_and_relabel (dir_and_old_label, new_label):
    replicator.umount (dir_and_old_label);

    device = replicator.get_device_by_label (dir_and_old_label);
    if device == None or device == "":
        print "Cannot find device with label %s" % dir_and_old_label
        sys.exit (1)

    replicator.relabel_device (device, new_label)

class progressBar:
    def __init__ (self):
        self.last_percent = 0
        self.last_state = replicator.STATE_INIT
        self.hashes_printed = -1

        self.state_text = {
            replicator.STATE_INIT         : "Initializing        ",
            replicator.STATE_SYNCING_ROOT : "Copying system image",
            replicator.STATE_SYNCING_BOOT : "Copying kernel image",
            replicator.STATE_POSTPROCESS  : "Preparing bootloader",
            replicator.STATE_FINISHED     : "Copying complete    ",
            replicator.STATE_ERROR        : "Error               "
            }
        self.textlen = len (self.state_text[replicator.STATE_INIT])

    def print_progress (self, percent, state):
        hashes_total = 40

        hashes_needed = int (hashes_total * (float (percent) / 100))
        
        if sys.stdout.isatty ():
            sys.stdout.write ('  ')
                
        for i in range (hashes_needed):
            sys.stdout.write ('#')

        for i in range (hashes_needed, hashes_total):
            sys.stdout.write (' ')

        sys.stdout.write ("(%3d%%) [%s]" % (percent, self.state_text[state]))
        
        if hashes_needed < hashes_total:
            for i in range (2 + hashes_total + 9 + self.textlen):
                sys.stdout.write ("\b")

        self.last_percent   = percent
        self.last_state     = state
        self.hashes_printed = hashes_needed

        sys.stdout.flush ()

    def finish (self):
        sys.stdout.write ("\n")
        
    def update_percent (self, percent):
        if percent <= self.last_percent:
            return

        self.print_progress (percent, self.last_state)
    
    def update_state (self, state):
        if state == self.last_state:
            return

        self.print_progress (self.last_percent, state)

def Usage ():
    print "stateless-bootstrap: - Stateless Linux client bootstrap tool\n\nUsage: stateless-bootstrap [--debug] [--reserve-root] [--reserve-boot]"

if __name__ == '__main__':
    import getopt
    import os
            
    if os.getuid () != 0:
        print "Please restart %s with root permissions!" % (sys.argv[0])
        sys.exit (1)

    debug_mode = False
    reserve_root_dir = None
    reserve_boot_dir = None
    
    class BadUsage: pass
    try:
        opts, args = getopt.getopt (sys.argv[1:],
                                    "dr:b:h",
                                    ["debug", "reserve-root", "reserve-boot", "help"])
        for opt, val in opts:
            if opt == '-d' or opt == '--debug':
                debug_mode = True

            if opt == '-r' or opt == '--reserve-root':
                reserve_root_dir = val
            
            if opt == '-b' or opt == '--reserve-boot':
                reserve_boot_dir = val

            if opt == '-h' or opt == '--help':
                Usage ()
                sys.exit (0)
    
    except (getopt.error, BadUsage):
        Usage ()
        sys.exit (1)

    if reserve_root_dir != None:
        if debug_mode:
            print "Unmounting %s and re-labelling as 'RESERVE_ROOT'" % reserve_root_dir
        unmount_and_relabel (reserve_root_dir, "RESERVE_ROOT");
    
    if reserve_boot_dir != None:
        if debug_mode:
            print "Unmounting %s and re-labelling as 'RESERVE_BOOT'" % reserve_boot_dir
        unmount_and_relabel (reserve_boot_dir, "RESERVE_BOOT");

    bootstrap = StatelessBootstrap (debug_mode)
    progress  = progressBar ()
    
    def on_state_change (bootstrap, state, progress):
        progress.update_state (state)

    def on_update (bootstrap, percent, progress):
        progress.update_percent (percent)
    
    def on_finished (bootstrap, success, progress):
        if success:
            progress.update_percent (100)
        progress.finish ()

    bootstrap.connect ("state-change", on_state_change, progress)
    bootstrap.connect ("update",       on_update,       progress)
    bootstrap.connect ("finished",     on_finished,     progress)
    bootstrap.run ()
